@@ -1,4 +1,4 @@
-// SPDX-License-Identifier: Apache-2.0
+// SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 /*
  * This file deals with setup of /cpus/ibm,powerpc-cpu-features dt
  *
@@ -827,7 +827,6 @@ static void add_cpufeatures_dependencies(struct dt_node *features)
 		deps_names = f->dependencies_names;
 		nr_deps = strcount(deps_names, " ") + 1;
 		dt_resize_property(&deps, nr_deps * sizeof(u32));
-		deps->len = nr_deps * sizeof(u32);
 
 		DBG("feature %s has %d dependencies (%s)\n", f->name, nr_deps, deps_names);
 		/*
@@ -943,6 +942,25 @@ void dt_add_cpufeatures(struct dt_node *root)
 			default:
 				assert(0);
 			}
+		} else if (is_power9c(version) &&
+                            (PVR_VERS_MAJ(version) == 1)) {
+                          /* P9C DD1.x */
+			switch (PVR_VERS_MIN(version)) {
+                        case 1:
+				/* Cumulus DD1.1 => Nimbus DD2.1 */
+				cpu_feature_cpu = CPU_P9_DD2_0_1;
+				break;
+			case 2:
+				/* Cumulus DD1.2 */
+				cpu_feature_cpu = CPU_P9_DD2_2;
+				break;
+			case 3:
+				/* Cumulus DD1.3 */
+				cpu_feature_cpu = CPU_P9_DD2_3;
+				break;
+			default:
+				assert(0);
+			}
 		} else {
 			assert(0);
 		}
