@@ -1,4 +1,4 @@
-// SPDX-License-Identifier: Apache-2.0
+// SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 /*
  * In-Memory Counters (IMC)
  * Sometimes called IMA, but that's also a different thing.
@@ -448,18 +448,18 @@ void imc_catalog_preload(void)
 static void imc_dt_update_nest_node(struct dt_node *dev)
 {
 	struct proc_chip *chip;
-	uint64_t *base_addr = NULL;
-	uint32_t *chipids = NULL;
+	__be64 *base_addr = NULL;
+	__be32 *chipids = NULL;
 	int i=0, nr_chip = nr_chips();
 	struct dt_node *node;
 	const struct dt_property *type;
 
 	/* Add the base_addr and chip-id properties for the nest node */
-	base_addr = malloc(sizeof(uint64_t) * nr_chip);
-	chipids = malloc(sizeof(uint32_t) * nr_chip);
+	base_addr = malloc(sizeof(u64) * nr_chip);
+	chipids = malloc(sizeof(u32) * nr_chip);
 	for_each_chip(chip) {
-		base_addr[i] = chip->homer_base;
-		chipids[i] = chip->id;
+		base_addr[i] = cpu_to_be64(chip->homer_base);
+		chipids[i] = cpu_to_be32(chip->id);
 		i++;
 	}
 
@@ -544,8 +544,10 @@ void imc_init(void)
 		return;
 
 	wait_xz_decompress(imc_xz);
-	if (imc_xz->status != OPAL_SUCCESS)
+	if (imc_xz->status != OPAL_SUCCESS) {
+		prerror("IMC: xz_decompress failed\n");
 		goto err;
+	}
 
 	/*
 	 * Flow of the data from PNOR to main device tree:
@@ -559,8 +561,10 @@ void imc_init(void)
 
 	/* Create a device tree entry for imc counters */
 	dev = dt_new_root("imc-counters");
-	if (!dev)
+	if (!dev) {
+		prerror("IMC: Failed to add an imc-counters root node\n");
 		goto err;
+	}
 
 	/*
 	 * Attach the new decompress_buf to the imc-counters node.
@@ -568,6 +572,7 @@ void imc_init(void)
 	 */
 	if (dt_expand_node(dev, imc_xz->dst, 0) < 0) {
 		dt_free(dev);
+		prerror("IMC: dt_expand_node failed\n");
 		goto err;
 	}
 
@@ -600,8 +605,10 @@ imc_mambo:
 	 * then out to band tools will race with ucode and end up getting
 	 * undesirable values. Hence pause the ucode if it is already running.
 	 */
-	if (pause_microcode_at_boot())
+	if (pause_microcode_at_boot()) {
+		prerror("IMC: Pausing ucode failed\n");
 		goto err;
+	}
 
 	/*
 	 * If the dt_attach_root() fails, "imc-counters" node will not be
@@ -610,6 +617,7 @@ imc_mambo:
 	 */
 	if (!dt_attach_root(dt_root, dev)) {
 		dt_free(dev);
+		prerror("IMC: Failed to attach imc-counter node to dt root\n");
 		goto err;
 	}
 
@@ -677,7 +685,7 @@ static int64_t opal_imc_counters_init(uint32_t type, uint64_t addr, uint64_t cpu
 		 * pdbar in specific scom ports. port_id are in
 		 * pdbar_scom_index[] and htm_scom_index[].
 		 */
-		phys_core_id = cpu_get_core_index(c);
+		phys_core_id = pir_to_core_id(c->pir);
 		port_id = phys_core_id % 4;
 
 		if (proc_chip_quirks & QUIRK_MAMBO_CALLOUTS)
@@ -754,7 +762,7 @@ static int64_t opal_imc_counters_init(uint32_t type, uint64_t addr, uint64_t cpu
 		if (!c)
 			return OPAL_PARAMETER;
 
-		phys_core_id = cpu_get_core_index(c);
+		phys_core_id = pir_to_core_id(c->pir);
 		port_id = phys_core_id % 4;
 
 		if (proc_chip_quirks & QUIRK_MAMBO_CALLOUTS)
@@ -832,7 +840,7 @@ static int64_t opal_imc_counters_start(uint32_t type, uint64_t cpu_pir)
 		 * Core IMC hardware mandates setting of htm_mode in specific
 		 * scom ports (port_id are in htm_scom_index[])
 		 */
-		phys_core_id = cpu_get_core_index(c);
+		phys_core_id = pir_to_core_id(c->pir);
 		port_id = phys_core_id % 4;
 
 		if (proc_chip_quirks & QUIRK_MAMBO_CALLOUTS)
@@ -894,7 +902,7 @@ static int64_t opal_imc_counters_stop(uint32_t type, uint64_t cpu_pir)
 		 * Core IMC hardware mandates setting of htm_mode in specific
 		 * scom ports (port_id are in htm_scom_index[])
 		 */
-		phys_core_id = cpu_get_core_index(c);
+		phys_core_id = pir_to_core_id(c->pir);
 		port_id = phys_core_id % 4;
 
 		if (proc_chip_quirks & QUIRK_MAMBO_CALLOUTS)
