@@ -1,4 +1,4 @@
-// SPDX-License-Identifier: Apache-2.0
+// SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 /*
  * Copyright 2019 IBM Corp.
  */
@@ -350,7 +350,6 @@ static void npu3_append_phandle(struct dt_node *dn, const char *name,
 	/* Need to append to the property */
 	len = prop->len + sizeof(*phandles);
 	dt_resize_property(&prop, len);
-	prop->len = len;
 
 	phandles = (uint32_t *)prop->prop;
 	phandles[len / sizeof(*phandles) - 1] = phandle;
@@ -904,7 +903,8 @@ static void npu3_create_phb(struct npu3 *npu)
 	assert(phb->dt_node);
 
 	list_head_init(&phb->virt_devices);
-	pci_register_phb(phb, OPAL_DYNAMIC_PHB_ID);
+	pci_register_phb(phb, npu3_get_opal_id(npu->chip_id,
+					       npu3_get_phb_index(npu->index)));
 	npu3_create_phb_slot(npu);
 	npu3_ioda_reset(phb, true);
 }
@@ -928,8 +928,6 @@ static void npu3_dev_init_hw(struct npu3_dev *dev)
 	reg = NPU3_NTL_MISC_CFG2(dev->index);
 	val = npu3_read(npu, reg);
 	val |= NPU3_NTL_MISC_CFG2_BRICK_ENABLE |
-	       NPU3_NTL_MISC_CFG2_NDL_TX_PARITY_ENA |
-	       NPU3_NTL_MISC_CFG2_NDL_PRI_PARITY_ENA |
 	       NPU3_NTL_MISC_CFG2_RCV_CREDIT_OVERFLOW_ENA;
 	npu3_write(npu, reg, val);
 }
@@ -1220,13 +1218,16 @@ static int64_t npu3_dev_salt(void *pvd, struct pci_cfg_reg_filter *pcrf,
 		return OPAL_PARAMETER;
 
 	/* The config register before this one holds CMD_REG */
-	pci_virt_cfg_read_raw(pvd, PCI_VIRT_CFG_NORMAL, pcrf->start - 4,
-			      4, &cmd_reg);
+	PCI_VIRT_CFG_NORMAL_RD(pvd, pcrf->start - 4, 4, &cmd_reg);
+	if (cmd_reg == 0xffffffff)
+		return OPAL_PARAMETER;
 
 	/* Check for another command in progress */
 	val = npu3_dev_ppe_sram_read(dev, OB_PPE_SALT_CMD);
-	if (GETFIELD(OB_PPE_SALT_CMD_READY, val))
+	if (GETFIELD(OB_PPE_SALT_CMD_READY, val)) {
+		NPU3DEVINF(dev, "SALT_CMD 0x%x: Not ready\n", cmd_reg);
 		return OPAL_BUSY;
+	}
 
 	val = OB_PPE_SALT_CMD_READY;
 	val = SETFIELD(OB_PPE_SALT_CMD_RW, val, write);
@@ -1242,7 +1243,7 @@ static int64_t npu3_dev_salt(void *pvd, struct pci_cfg_reg_filter *pcrf,
 
 	while (GETFIELD(OB_PPE_SALT_CMD_READY, val)) {
 		if (tb_compare(mftb(), timeout) == TB_AAFTERB) {
-			NPU3DEVINF(dev, "SALT_CMD 0x%x: timeout\n", cmd_reg);
+			NPU3DEVINF(dev, "SALT_CMD 0x%x: Timeout\n", cmd_reg);
 			return OPAL_BUSY;
 		}
 
@@ -1250,7 +1251,7 @@ static int64_t npu3_dev_salt(void *pvd, struct pci_cfg_reg_filter *pcrf,
 	}
 
 	if (GETFIELD(OB_PPE_SALT_CMD_ERR, val))
-		NPU3DEVINF(dev, "SALT_CMD 0x%x: error\n", cmd_reg);
+		NPU3DEVINF(dev, "SALT_CMD 0x%x: Error\n", cmd_reg);
 
 	if (!write)
 		*data = GETFIELD(OB_PPE_SALT_CMD_DATA, val);
@@ -1290,7 +1291,7 @@ static uint32_t npu3_cfg_populate_vendor_cap(struct npu3_dev *dev,
 	PCI_VIRT_CFG_INIT_RO(pvd, start + 0xc, 1, npu3_chip_dev_index(dev));
 
 	/* SALT registers */
-	PCI_VIRT_CFG_INIT_RO(pvd, start + 0x10, 4, 0);
+	PCI_VIRT_CFG_INIT(pvd, start + 0x10, 4, 0xffffffff, 0, 0);
 	PCI_VIRT_CFG_INIT_RO(pvd, start + 0x14, 4, 0);
 
 	pci_virt_add_filter(pvd, start + 0x14, 4,
@@ -1466,13 +1467,18 @@ static void npu3_dt_add_props(struct npu3 *npu)
 	dt_add_property_cells(dn, "clock-frequency", 0x200, 0);
 
 	dt_add_property_strings(dn, "device_type", "pciex");
-	/* To the OS, npu2 and npu3 are both ibm,ioda2-npu2-phb */
+
+	/*
+	 * To the OS, npu2 and npu3 are both ibm,ioda2-npu2-phb. The added
+	 * ibm,ioda3-npu3-phb allows for possible quirks.
+	 */
 	dt_add_property_strings(dn, "compatible",
 				"ibm,power9-npu-pciex",
-				"ibm,ioda2-npu2-phb");
+				"ibm,ioda2-npu2-phb",
+				"ibm,ioda2-npu3-phb");
 
 	dt_add_property_cells(dn, "ibm,phb-index",
-			      dt_prop_get_u32(npu->dt_node, "ibm,phb-index"));
+			      npu3_get_phb_index(npu->index));
 	dt_add_property_cells(dn, "ibm,phb-diag-data-size", 0);
 	dt_add_property_cells(dn, "ibm,opal-num-pes", NPU3_MAX_PE_NUM);
 	dt_add_property_cells(dn, "ibm,opal-reserved-pe", NPU3_RESERVED_PE_NUM);
